package com.ruoyi.system.domain;

import cn.afterturn.easypoi.excel.annotation.Excel;
import com.ruoyi.common.core.domain.BaseEntity;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.List;

/**
 * 船舶信息对象 ship_recognition
 *
 * @author ruoyi
 * @date 2021-08-24
 */
@EqualsAndHashCode(callSuper = true)
@Data
public class ShipRecognition extends BaseEntity {
    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    private Long id;

    /**
     * AIS船舶距离, 单位:m
     */
//    @Excel(name = "船舶距离, 单位:m")
//    private BigDecimal aisDist;

    /**
     * AIS吃水, 单位:m
     */
//    @Excel(name = "吃水, 单位:m")
//    private BigDecimal aisDrt;

    /**
     * AIS目的地
     */
    @Excel(name = "目的地")
    private String aisDst;

    /**
     * 9位码。例如412119710, 0或null表示AIS未开启
     */
    @Excel(name = "MMSI")
    private Long aisMmsi;

    /**
     * AIS船长, 单位:m
     */
//    @Excel(name = "船长, 单位:m")
//    private Long aisShipLen;

    /**
     * 可能是中文,AIS船名,为拼音。例如SU SU HUO 009
     */
    @Excel(name = "船名")
    private String aisShipName;

    private String shipRegionType;
    private String orgName;

    /**
     * AIS船速, 单位:m/s
     */
    @Excel(name = "船速, 单位:m/s")
    private BigDecimal aisShipSpd;

    /**
     * AIS船舶类型
     */
    @Excel(name = "船舶类型")
    private String aisShipType;

    /**
     * AIS船宽, 单位:m
     */
    @Excel(name = "船宽, 单位:m")
    private Long aisShipWidth;

    /**
     * 尺寸异常
     */
//    @Excel(name = "尺寸异常")
//    private Long isAbnormalSize;

    /**
     * AIS纬度,单位:度
     */
    @Excel(name = "纬度,单位:度")
    private BigDecimal latitude;

    /**
     * 黑烟:度数
     */
    @Excel(name = "黑烟:度数")
    private Long rcgSoot;

    private List<Long> rcgSoots;

    /**
     * 船舶过闸闸次
     */
//    @Excel(name = "船舶过闸闸次")
//    private Long lockSeq;

    /**
     * AIS经度,单位:度
     */
    @Excel(name = "经度,单位:度")
    private BigDecimal longitude;

    /**
     * 图像识别得到的载货类型。例如:钢筋,砂石。
     */
    @Excel(name = "图像识别载货类型")
    private String rcgCargoType;

    /**
     * 图像识别得到的船舶载重吨位, 单位:t
     */
//    @Excel(name = "图像识别船舶载重吨位, 单位:t")
//    private BigDecimal rcgDeadweightTon;

    /**
     * 图像识别得到的,船舶距离相机直线距离。单位:m
     */
//    @Excel(name = "图像识别船舶距离相机直线距离。单位:m")
//    private BigDecimal rcgDist;

    /**
     * 图像识别得到的船舶吃水, 单位:m
     */
//    @Excel(name = "图像识别船舶吃水, 单位:m")
//    private BigDecimal rcgDrt;

    /**
     * 是否为船队。0 否,即单机船; 1 是,即为船队
     */
//    @Excel(name = "是否为船队。0 否,即单机船; 1 是,即为船队")
//    private Long rcgIsFleet;

    /**
     * UNKNOWN 未知, EMPTY_LOAD 空载, HEAVY_LOAD 重载, OVER_LOAD 超载
     */
//    @Excel(name = "UNKNOWN 未知, EMPTY_LOAD 空载, HEAVY_LOAD 重载, OVER_LOAD 超载")
//    private String rcgLoadSts;

    /**
     * 图像识别得到的船舶水面上高度,单位:m
     */
    @Excel(name = "图像识别船舶水面上高度,单位:m")
    private BigDecimal rcgShipHeightAboveWater;

    /**
     * 图像识别得到的船长,单位:m
     */
    @Excel(name = "图像识别船长,单位:m")
    private BigDecimal rcgShipLen;

    /**
     * 图像识别得到的船名, 中文, 如: 苏宿货009
     */
    @Excel(name = "图像识别船名")
    private String rcgShipName;

    /**
     * 图像识别得到的速度,带符号,单位m/s
     */
    @Excel(name = "图像识别速度,单位m/s")
    private BigDecimal rcgShipSpd;

    /**
     * 图像识别得到的船舶类型。例如: 货船,集装箱船,危化品船
     */
//    @Excel(name = "图像识别船舶类型")
//    private String rcgShipType;

    /**
     * 图像识别得到的船宽,单位:m
     */
//    @Excel(name = "图像识别船宽,单位:m")
//    private BigDecimal rcgShipWidth;

    /**
     * 船籍港
     */
//    @Excel(name = "船籍港")
//    private String regportName;

    /**
     * 抓拍图片路径
     */
//    @Excel(name = "抓拍图片路径")
//    private String snapImPath;

    /**
     * 抓拍地点
     */
    @Excel(name = "抓拍地点")
    private String snapPos;

    /**
     * 抓拍时间
     */
    private String snapTime;

    /**
     * 抓拍时间统一格式: 2020-12-12 12:00:11
     */
    @Excel(name = "抓拍时间")
    private String snapTimeFmt;

    /**
     * 状态
     */
    @Excel(name = "状态")
    private String status;

    /**
     * 船名特征信息
     */
//    @Excel(name = "船名特征信息")
//    private String textFeatureLabel;

    /**
     * 船名识别原图url
     */
//    @Excel(name = "船名识别原图url")
//    private String textImgUrl;

    /**
     * 图像识别得到的船舶详细类型
     */
//    @Excel(name = "图像识别得到的船舶详细类型")
//    private String rcgShipTypeDetail;

    /**
     * useWarn: true, 使用外部输入预警信息
     */
//    @Excel(name = "使用外部输入预警信息")
//    private Long useWarn;

    /**
     * 外部输入报警信息详情
     */
//    @Excel(name = "外部输入报警信息详情")
//    private String warnLabelList;

    /**
     * 二维码识别内容
     */
//    @Excel(name = "二维码识别内容")
//    private String qrcode;

    /**
     * 船名方向
     */
//    @Excel(name = "船名方向")
//    private String shipNameDirection;

    @Excel(name = "正确率更高的船名")
    private String shipNameValid;

    @Excel(name = "黑烟度数特征信息")
    private String sootFeatureLabel;

    @Excel(name = "黑烟度数原图url")
    private String sootImgUrl;

    @Excel(name = "所有图片")
    private String allImgUrl;

    private Integer illegalStatus;
    private Integer uploadFlag;

    private String shipName;

    private String monitorPointName;

    private String mmsi;

    private String filterMmsi;

    private String deviceId;

    private String deviceName;
}
